@@ -99,6 +99,12 @@ class FilterHeader(QHeaderView):
         # Add encode text action
         encode_action = context_menu.addAction("Encode Text")
         
+        # Add predict action
+        predict_action = context_menu.addAction("Predict Column")
+        
+        # Add load model action
+        load_model_action = context_menu.addAction("Load Model & Predict")
+        
         context_menu.addSeparator()
         filter_action = context_menu.addAction("Filter...")
         
@@ -156,6 +162,19 @@ class FilterHeader(QHeaderView):
                     if logical_index < len(current_tab.current_df.columns):
                         column_name = current_tab.current_df.columns[logical_index]
                         self.main_window.encode_text(column_name)
+        elif action == predict_action:
+            # Call the predict_column method on the main window
+            if self.main_window and hasattr(self.main_window, "predict_column"):
+                # Get the column name from the table (if it has a current dataframe)
+                current_tab = self.main_window.get_current_tab()
+                if current_tab and hasattr(current_tab, "current_df") and current_tab.current_df is not None:
+                    if logical_index < len(current_tab.current_df.columns):
+                        column_name = current_tab.current_df.columns[logical_index]
+                        self.main_window.predict_column(column_name)
+        elif action == load_model_action:
+            # Call the load_and_apply_model method on the main window
+            if self.main_window and hasattr(self.main_window, "load_and_apply_model"):
+                self.main_window.load_and_apply_model()
         elif action == count_rows_action:
             # Get the current tab and show row count
             current_tab = self.main_window.get_current_tab()
